#!/usr/bin/perl -w
use strict;

use Gtk;
use Gtk::Atoms;
use Gtk::HandyCList;
use Gtk::Gdk;
use Data::Dumper;
use Frontier::Client;

my $url = 'http://localhost:8000/RPC2';
my $current_user = 'dmo';
my $current_password = 'dmo';

my $XML =  Frontier::Client->new( url => $url);

init Gtk;

my $window;
my $statusbar; ## Grrr.  I hate globals.
create_main_window();


main Gtk;

sub create_main_window 
{
	$window = new Gtk::Window('toplevel');
	$window->set_name("Scraps Client");
	$window->set_title("Scraps Client");
	$window->set_usize(600,400);

	$window->signal_connect("destroy" => \&Gtk::main_quit);
	$window->signal_connect("delete_event" => \&Gtk::false);

	my $vbox = new Gtk::VBox(0,0);
	$window->add($vbox);
	$vbox->show;
	
#	my $pack = new Gtk::Packer;
#	$window->add($pack);
#	$pack->show;
	
	my $menubar = new Gtk::MenuBar;
#	$pack->add_defaults($menubar, -top, -north, [ -fill_x, -expand]);
	$vbox->pack_start($menubar,0,0,0);
	$menubar->show;
	
	my $menuitem = new Gtk::MenuItem("File");
	$menubar->append($menuitem);
	$menuitem->show;

## TODO: Figure out toolbar.
#	my $toolbar = new Gtk::Toolbar('horizontal','both');
#	$toolbar->set_space_style('line');
#	$toolbar->set_button_relief("none");
#	my $button = $toolbar->append_item("New Scrap","Create a new scrap",
#	$vbox->pack_start($toolbar);
#	$toolbar->show;

	## TODO add a frame here
	my $frame = new Gtk::Frame('Search Scraps');
	#$pack->add_defaults($frame,-top,-north, [-fill_x, -fill_y, -expand]);
	
	$vbox->pack_start($frame,1,1,2);
	$frame->set_border_width(5);
	$frame->show;
	
	my $frame_vbox = new Gtk::VBox(0,5);
	$frame_vbox->set_border_width(5);
	$frame->add($frame_vbox);
	$frame_vbox->show;
	
	## add hbox to the frame containing search box and button
	my $frame_hbox = new Gtk::HBox(0,5);
	$frame_vbox->pack_start($frame_hbox,0,0,0);
	$frame_hbox->show;
	
	my $search_label = new Gtk::Label "Search keywords:";
	$frame_hbox->pack_start($search_label,0,0,0);
	$search_label->show;


	my $search_text = new Gtk::Entry;
	$search_text->set_max_length(80);
	$frame_hbox->pack_start($search_text,0,0,0);
	$search_text->show;
	
	my $search_button = new Gtk::Button "Search!";
	$frame_hbox->pack_start($search_button,0,0,0);
	$search_button->show;
	## add scrolled window.
	my $scrolled_win = new Gtk::ScrolledWindow(undef,undef);
	$scrolled_win->set_policy( 'always', 'automatic' );
	$frame_vbox->pack_start( $scrolled_win, 1, 1, 0 );
	$scrolled_win->show;
	
	## add clist to scrolled window.
	my $list = Gtk::HandyCList->new( qw(id Title Description Modified) );
	$list->hide(qw(id));
	$list->sizes( id=> 0,Title=> "30%",Description=>"50%",Modified=>100);
	$list->sortfuncs( id => "alpha", Title => "alpha", Description => "alpha", Modified => "number");
	
	$scrolled_win->add($list);
	$list->show;
	
	$search_button->signal_connect('clicked', \&cb_search_clicked, $search_text,$list);
	$list->signal_connect('button_press_event', \&cb_listrow_clicked);


	## Add status bar.  $statusbar is global.
	$statusbar = new Gtk::Statusbar;
	$vbox->pack_end($statusbar, 0,0,0);
	$statusbar->show;
	
	$window->show;
}

sub cb_search_clicked
{
	my $self = shift;
	my $entry = shift;
	my $list = shift;
	$list->clear;

	my $keywords = $entry->get_text();
	## Now, we replace spaces in quoted keywords with ::, and
	## remove quotes.  Then, split on whitespace, then replace the :: chars with
	## whitespace.  Ugh.
	my $s_crit = {};
	$keywords =~  s/\"([^"]+)\"/my $x = $1; $x =~ s|\s+|::|g; $x;/eg;
	my @keywords = split(/\s/,$keywords);
	$s_crit->{and} = [ map { $_ =~ s/::/ /g; { keyword => $_ } } @keywords];
	my $scrap_data;
	set_status_message("Fetching data from server");
	eval {
		$scrap_data = $XML->call('scraps.search',$current_user,$current_password,{ query => $s_crit });
	};
	if($@){
		## TODO add error message popup here
		print STDERR "Error fetching data!: $@";
		return 0;
	}
	
	if(!scalar(@$scrap_data)) {
		set_status_message("No results.");
		return 1;
	}
	set_status_message("Search successful.");
	
	## Now, remap names of data fields in returned refs to be
	## same as our rows (namely id, Description, Title, and Modified
	map { 
		$_->{Description} = $_->{description};
		delete($_->{description});
		$_->{Title} = $_->{title};
		delete($_->{title});
		$_->{Modified} = $_->{date_modified};
		delete($_->{date_modified});
	} @$scrap_data;

	$list->data(@$scrap_data);

	$list->refresh;

	return 1;
}

sub cb_listrow_clicked
{
	my $self = shift || return;
	my $press_event = shift || return;
	
	return 0 unless ('2button_press' eq $press_event->{'type'});

	#print STDERR Dumper $self->selection->{(keys %{$self->selection})[0]};
	my $scrap_data;
	my $selected =  $self->selection->{(keys %{$self->selection})[0]};
	return 0 unless (defined($selected->{id}));
	set_status_message("Fetching data from server");
	eval {
		$scrap_data = $XML->call('scraps.fetchScrap',$current_user,$current_password,$selected->{id});
	};
	if($@){
		## TODO add error message popup here
		print STDERR "Error fetching data!: $@";
		return 0;
	}
	
	if(!defined($scrap_data)) {
		set_status_message("No results.");
		return 1;
	}
	set_status_message("Search successful.");

	my $win = create_scrap_window($scrap_data);


	return 1;
}

sub set_status_message 
{
	my $msg = shift || return;

	return 0 if(!defined($statusbar));
	
	if($statusbar->messages) {
		$statusbar->pop(1);
	}
	$statusbar->push(1,$msg);
}

sub create_scrap_window
{
	my $scrap_data = shift;
	
	my $window = new Gtk::Window('dialog');
	$window->set_name("scrap window");
	$window->set_title("View Scrap Data");
	$window->set_usize(400,400);

	$window->signal_connect("destroy" => sub { destroy $window });
	$window->signal_connect("delete_event" => \&Gtk::false);


	my $vbox = new Gtk::VBox(0,0);
	$window->add($vbox);
	$vbox->show;

	## Table w/ 5 rows, 2 columns, non-homogenous.
	my $table = new Gtk::Table(5,2,0);
	$table->set_border_width(5);
	$vbox->pack_start($table,0,0,0);
	$table->show;

	my $page_title = new Gtk::Label("Scrap View");
	$page_title->set_justify('left');
	my $title_style = new Gtk::Style;
	my $title_font = Gtk::Gdk::Font->load(
		"-*-helvetica-bold-r-normal-*-18-120-*-*-p-*-iso8859-1"
	);
	$title_style->font($title_font);
	$page_title->set_style($title_style);
	$table->attach_defaults($page_title,0,2,0,1);
	$page_title->show;

	my $title_l = new Gtk::Label("Title:");
	$table->attach_defaults($title_l,0,1,1,2);
	$title_l->show;

	my $title_d = new Gtk::Entry();
	$title_d->set_text($scrap_data->{title});
	$title_d->set_max_length(80);
	$table->attach_defaults($title_d,1,2,1,2);
	$title_d->show;
	
	$window->show;
	return $window;
}
