#!perl

package Scraps::Client::Handler;

sub new 
{
	my $class = shift;

	my $self = {};
	bless($self,$class);

	## TODO: make this an array-based hash.
	$self->{CGI} = undef;
	$self->{RPCHANDLE} = undef;
	$self->{USERNAME} = undef;
	$self->{PASSWORD} = undef;
	$self->{OUTPUT} = undef;
	$self->{HEADER} = undef;
	return $self;
}

sub get_output
{
	my $self = shift;
	return $self->{OUTPUT};
}

sub get_header
{
	my $self = shift;
	if($self->{HEADER}) {
		return $self->{HEADER};
	} else {
		return $self->{CGI}->header;
	}
}

=pod

=item B<get_template($filename,$allow_global)>

This is a wrapper around HTML::Template->new() to help clean things up.
$filename should be the filename of the desired template, relative to 
$Scraps::Client::Config::template_dir.  
$allow_global is whether or not to allow variables declared outside a loop
be allowed inside the loop.  Default is no.

=cut

sub get_template
{
	my $self = shift;
	my $filename = shift;
	my $allow_global = shift || 0;

	return HTML::Template->new(
		path => [ $Scraps::Client::Config::template_dir ],
		filename => $filename,
		global_vars => $allow_global
	);

}

=pod

=item B<notice(msg))>

Sets $self->{OUTPUT} to a generic error with the given message.  If
optional argument given, will log to server error log.

=cut

sub notice 
{
	my $self = shift;
	my $msg  = shift;
	my $log  = shift || 0;

	if($log) {
		## Carp seems to be broken, so we'll use this instead
		my ($package, $filename, $line) = caller();
		my $date = scalar(localtime(time));
		print STDERR "[$date] $msg at $filename line $line.\n";
	}

	my $t = $self->get_template('generic_error.tmpl');
	$t->param(
		reason => $msg
	);
	$self->{OUTPUT} = $t->output();

	return undef;
}

=pod

=item B<error($msg)>

Sets $self->{OUTPUT} to a generic error with the given message.  

=cut

sub error 
{
	my $self = shift;
	my $msg  = shift;

	return $self->notice($msg,1);
}

=item B<message($msg)>

Returns a generic message screen with the given message.

=cut

sub message 
{
	my $self = shift;
	my $msg  = shift;

	my $t = $self->get_template('generic_message.tmpl');
	$t->param(
		message => $msg,
		script => $self->get_cgi()->url()
	);
	$self->{OUTPUT} = $t->output();
}

=pod

=item B<redirect()>

Redirects to given URL, with optional cookie setting.

=cut

sub redirect 
{
	my $self = shift;
	my %args = @_;
	my $q = $self->get_cgi();

	if(exists($args{'cookie'})) {
		$self->{HEADER} = $q->redirect(
			-location => $q->url . $args{'params'},
			-cookie   => $q->cookie(
				-name   => $Scraps::Client::Config::cookieName,
				-value  => $args{'cookie'},
#				-path   => $Scraps::Client::Config::web_path
			)
		);
	} else {
		$self->{HEADER} = $q->redirect(
			-location => $q->url . $args{'params'}
		);
	}
	$self->{OUTPUT} = "";
}

##
## List valid types.
##
sub get_valid_types
{
	my $self = shift;
	my $selected = shift || undef;
	my $blort = [
		{ name => 'Plain text', value => 'text', selected => 0 },
		{ name => 'URL', value => 'url', selected => 0 },
		{ name => 'Stored Query', value => 'query', selected => 0 },
	];

	if(defined($selected)) {
		foreach my $line (@$blort) {
			if($line->{value} eq $selected) {
				$line->{selected} = 1;
				goto done;
			}
		}
	}
done:
	return $blort;
	
}

sub check_permission
{
	my $self = shift;
	my $what = shift;
	## TODO: Will autovivification be a problem here?  Make this an array-based hash?   
	return 1 if($self->{PERMISSIONS}->{$what});
	return 1 if($self->{PERMISSIONS}->{all});
	return 0;
}


##
## Abstract methods - must be implemented by subclass.
##
sub has_permission
{
	my $self = shift;
	$self->{OUTPUT} = "Someone needs to write a has_permission() method for " . ref($self) . "!";
	return undef;
}

sub process
{
	my $self = shift;
	$self->{OUTPUT} = "Someone needs to write a process() method for " . ref($self) . "!";
	return undef;
}


##
## Generic get/set methods.
sub set_username
{
	my $self = shift;
	$self->{USERNAME} = shift;
	return $self;
}
sub get_username
{
	my $self = shift;
	return $self->{USERNAME};
}
sub set_password
{
	my $self = shift;
	$self->{PASSWORD} = shift;
	return $self;
}
sub get_password
{
	my $self = shift;
	return $self->{PASSWORD};
}
sub set_cgi
{
	my $self = shift;
	$self->{CGI} = shift;
	return $self;
}
sub get_cgi
{
    my $self = shift;
    return $self->{CGI};
}
sub set_rpchandle
{
	my $self = shift;
	$self->{RPCHANDLE} = shift;
	return $self;
}
sub get_rpchandle
{
    my $self = shift;
    return $self->{RPCHANDLE};
}

1;
