#!perl

package Scraps::Client::Handler::addscrap;

use base qw(Scraps::Client::Handler);
use Data::Dumper;

sub process
{
	my $self = shift;
	my $q = $self->get_cgi();
	my $x = $self->get_rpchandle();

	my $tmpl; 
	if(!$q->param('submit')) {
		$tmpl = $self->get_template("addscrap.tmpl");
		$tmpl->param(
			'page_title' => 'Add Scrap',
			'op' => 'addscrap',
			'script' => $q->url(),
			## TODO: the following should be auto-generated from the data
			##       types accepted by the server.
			## TODO: make the server list accepted data types.
			'data_type' => $self->get_valid_types(),
		);
		$self->{OUTPUT} = $tmpl->output();
		return;
	}	

	## Otherwise, we should have a scrap here.  Build our struct and prepare
	## to send it.
	## This should be as dumb as possible, as we want all validation of 
	## data to be done on the server.
	## TODO: think about a generic Scrap object, using array-based pseudohashes

	## First, we need to munge spaces to deal with quoted keywords.
	my $keywords = $q->param('keywords');
	$keywords =~  s/\"([^"]+)\"/my $x = $1; $x =~ s|\s+|::|g; $x;/eg;
	my @keywords = split(/\s+/,$keywords);
	map { $_ =~ s/::/ /g; } @keywords;
	my $scrap = {
		title => $q->param('title'),
		creator => {
			name => $q->param('creator_name'),
			email => $q->param('creator_email'),
		},
		description => $q->param('description'),
		keywords => \@keywords,
		data => {
			type => $q->param('data_type') || '',
			data => $q->param('data_data') || '',
		},
	};
	my $result;
	eval { 
		$result = $x->call('scraps.newScrap',$self->get_username(),$self->get_password(),$scrap);
	};
	if($@) {
		my ($errstring) = $@ =~ /fault code \d+: (.*)$/;
		$tmpl = $self->get_template("addscrap.tmpl");
		$tmpl->param(
			'page_title' => 'Add Scrap',
			'op' => 'addscrap',
			'error_message' => $errstring,
			'script' => $q->url(),
			## TODO: the following should be auto-generated from the data
			##       types accepted by the server.
			## TODO: make the server list accepted data types.
			title => $q->param('title'),
			creator_name => $q->param('creator_name'),
			creator_email => $q->param('creator_email'),
			description => $q->param('description'),
			keywords => $q->param('keywords'),
			data_type => $self->get_valid_types($q->param('data_type')),
			data_data => $q->param('data_data'),
		);
		$self->{OUTPUT} = $tmpl->output();
		return;
	}
	my $scrap_id = $result->{id};

	$self->redirect( params => "?op=viewscrap;id=$scrap_id" );
}

sub has_permission
{
	my $self = shift;
	## TODO: check permissions?
	return 1;
}

1;
