#!perl

package Scraps::Client::Handler::editscrap;
use base qw(Scraps::Client::Handler);

sub process
{
	my $self = shift;
	my $q = $self->get_cgi();
	my $x = $self->get_rpchandle();
	my $tmpl; 

	if(!$q->param('id')) {
		return $self->notice("You can't edit a scrap without giving an ID!");
	}

	if(!$q->param('submit')) {
		my ($rc,$scrap) = $self->fetch_orig_data($q->param('id'));
		return if ($rc);
		
		$tmpl = $self->get_template("editscrap.tmpl");
		$tmpl->param(
			'page_title' => 'Edit Scrap',
			'op' => 'editscrap',
			'script' => $q->url(),
			'title' => $scrap->{title},
			'id' => $scrap->{id},
			'keywords' => join(" ",@{$scrap->{keywords}}),
			'description' => $scrap->{description},
			'creator_name' => $scrap->{creator}->{name},
			'creator_email' => $scrap->{creator}->{email},
			'data_data' => $scrap->{data}->{data},
			## TODO: the following should be auto-generated from the data
			##       types accepted by the server.
			## TODO: make the server list accepted data types.
			'data_type' => $self->get_valid_types($scrap->{data}->{type}),
		);
		$self->{OUTPUT} = $tmpl->output();
		return;
	}	

	## Otherwise, we should have a scrap here.  Build our struct and prepare
	## to send it.
	## This should be as dumb as possible, as we want all validation of 
	## data to be done on the server.
	## TODO: think about a generic Scrap object, using array-based pseudohashes

	## First, we need to munge spaces to deal with quoted keywords.
	my $keywords = $q->param('keywords');
	$keywords =~  s/\"([^"]+)\"/my $x = $1; $x =~ s|\s+|::|g; $x;/eg;
	my @keywords = split(/\s+/,$keywords);
	map { $_ =~ s/::/ /g; } @keywords;
	my $scrap = {
		title => $q->param('title'),
		description => $q->param('description'),
		keywords =>  \@keywords,
		contributor => [
			{ name => $q->param('contributor_name'),
			  email =>  $q->param('contributor_email'),
			  note =>  $q->param('contributor_note')
			}
		],
		data => {
			type => $q->param('data_type') || '',
			data => $q->param('data_data') || '',
		},
	};
	my $result;
	eval { 
		$result = $x->call('scraps.saveScrap',$self->get_username(),$self->get_password(),$q->param('id'),$scrap);
	};
	if($@) {
		## Eek!  We're fetching more data from the remote server!
		my ($rc,$old_scrap) = $self->fetch_orig_data($q->param('id'));
		return if ($rc);
		$tmpl = $self->get_template("editscrap.tmpl");
		$tmpl->param(
			'page_title' => 'Edit Scrap',
			'op' => 'editscrap',
			'id' => $old_scrap->{id},
			'error_message' => "An error occurred: $@",
			'script' => $q->url(),
			## TODO: the following should be auto-generated from the data
			##       types accepted by the server.
			## TODO: make the server list accepted data types.
			title => $q->param('title'),
			creator_name => $old_scrap->{creator}->{name},
			creator_email => $old_scrap->{creator}->{email},
			description => $q->param('description'),
			keywords => $q->param('keywords'),
			data_type => $self->get_valid_types($q->param('data_type')),
			data_data => $q->param('data_data'),
			contributor_name => $q->param('contributor_name'),
			contributor_email =>  $q->param('contributor_email'),
			contributor_note =>  $q->param('contributor_note')
		);
		$self->{OUTPUT} = $tmpl->output();
		return;
	}
	$self->redirect( params => "?op=viewscrap;id=" . $q->param('id') );
}

sub has_permission
{
	my $self = shift;
	## TODO: check permissions?
	return 1;
}

sub fetch_orig_data
{
	my $self = shift;
	my $id = shift;

	my $scrap;
	eval { 
		$scrap = $self->get_rpchandle()->call('scraps.fetchScrap',$self->get_username(),$self->get_password(),$id);
	};
	if($@) {
		return (1,$self->notice($@));
	}
	## Otherwise, we got our scrap.
	
	## Need to munge keywords to get spaces back in properly.
	map { $_ = qq("$_") if (/\s/); } @{$scrap->{keywords}};

	return (0,$scrap);
}

1;
