#!/perl

package Scraps::User;
use Scraps::Errors;
use Digest::MD5 qw(md5_hex);

=pod

scraps.user.verify(username,password)

This is basically a no-op, as the username/password must be valid to get
to this point.  If it returns, it's a valid username/password. If not, 
you get an error from the server and this never actually got called.

=cut

sub verify 
{
	return(Scraps::SUCCESS,"Success");
}

=pod 

scraps.user.add(username, password, new_username, new_password)
	- adds user with specified username/password.  
	- may want to extend to add user metadata (full name and email addr)

=cut

sub add 
{
	my $username = shift;
	my $db = shift;
	my $new_username = shift || '';
	my $new_password = shift || '';

	if($new_username =~ m/^\s*$/) {
		return (Scraps::INVALID_DATA,"Username is invalid");
	}
 	if($new_password =~ m/^\s*$/) {
		return (Scraps::INVALID_DATA,"Username is invalid");
	}

	my $sth = $db->prepare(qq{SELECT username from user where username = ?});
	$sth->execute($new_username);
	if(defined($sth->fetchrow_arrayref())) {
		return (Scraps::INVALID_DATA,"Username already exists");
	}

	$sth = $db->prepare(qq{INSERT INTO user (username,password) VALUES (?,?)});
	my $rv = $sth->execute($new_username, md5_hex($new_password));

	if($rv == 1) {
		return (Scraps::SUCCESS,'Success');
	}

	return(Scraps::INTERNAL_ERROR,"Couldn't add user");

}

=pod 

scraps.user.changePassword(username,password, target_username, new_password)
	- changes password for given user.
	- assumption that calling program will validate password for
	  correctness *ie: do the double-entry bit*
	- errors returned:

=cut

sub changePassword 
{
	my $username = shift;
	my $db = shift;
	my $target_username = shift;
	my $new_password = shift;

	if($target_username =~ m/^\s*$/) {
		return (Scraps::INVALID_DATA,"Username is invalid");
	}
 	if($new_password =~ m/^\s*$/) {
		return (Scraps::INVALID_DATA,"New password is invalid");
	}
	## TODO: could also apply password quality rules here.

	## Right now, a user is only allowed to change their own password
	## this will go away when we have a permissions hierarchy
	if($username ne $target_username) {
		return (Scraps::INVALID_PERMISSION,"Can only change password for self");
	}

	my $sth = $db->prepare(qq{SELECT username from user where username = ?});
	$sth->execute($target_username);
	if(!$sth->fetchrow_arrayref()) {
		return (Scraps::INVALID_DATA,"Username doesn't exist");
	}

	$sth = $db->prepare(qq{UPDATE user SET password = ? WHERE username = ?});
	my $rv = $sth->execute(md5_hex($new_password),$target_username);

	if($rv == 1) {
		return (Scraps::SUCCESS,'Success');
	}

	return(Scraps::INTERNAL_ERROR,"Couldn't add user");


}

=pod 

scraps.user.remove(username,password, remove_username)
	- remove user

=cut

sub remove 
{
	my $username = shift;
	my $db = shift;
	my $remove_username = shift;

	if($remove_username =~ m/^\s*$/) {
		return (Scraps::INVALID_DATA,"Username is invalid");
	}
 	if($remove_username eq $username) {
		return (Scraps::INVALID_DATA,"Cannot remove self!");
	}

	my $sth = $db->prepare(qq{SELECT username from user where username = ?});
	$sth->execute($remove_username);
	if(!$sth->fetchrow_arrayref()) {
		return (Scraps::INVALID_DATA,"Username does not exist");
	}

	$sth = $db->prepare(qq{DELETE FROM user WHERE username = ?});
	my $rv = $sth->execute($remove_username);

	if($rv == 1) {
		return (Scraps::SUCCESS,'Success');
	}

	return(Scraps::INTERNAL_ERROR,"error removing user");

}

sub list 
{
	my $username = shift;
	my $db = shift;

	my @users;

	my $sth = $db->prepare(qq{SELECT username from user});
	$sth->execute();
	while(defined($ary = $sth->fetchrow_arrayref())) {
		push @users, { username => $ary->[0] } ;
	}
	return (Scraps::SUCCESS,\@users);
}

1;
