CREATE DATABASE IF NOT EXISTS scraps;
grant all on scraps.* to 'scraps@localhost' identified by 'scraps';
use scraps;


-- This table stores the scrap data
CREATE TABLE IF NOT EXISTS scrap (
	id char(32) NOT NULL,
	title varchar(255) NOT NULL,
	creator_name varchar(255),
	creator_email varchar(255),
	date_created   datetime,
	date_modified  datetime,
	date_accessed  datetime,
	date_imported  datetime,
-- TEXT type is max size 2^16.  May want MEDIUMTEXT or LONGTEXT
	description	   text,
	keywords	text,
	datatype	varchar(255) DEFAULT 'text',
	data	mediumtext
);

-- m-n relation of contributor data to scraps.  No effort made to ensure 
-- uniqueness of name/email pairs, as each contribution here is specific to 
-- one scrap.  The table is only used b/c you can have several contributors
-- to one scrap.
-- However, we shall make the email address and scrap_id fields together\
-- be a key.  This should allow individual contributors to only be listed once
-- per scrap.
CREATE TABLE IF NOT EXISTS contributor (
	id int(10) AUTO_INCREMENT PRIMARY KEY, -- only used internally.
	scrap_id char(32),
	name varchar(255),
	email varchar(255),
	contrib_date datetime,
	data	text
);

-- This table stores the usernames and the associated passwords
CREATE TABLE IF NOT EXISTS user (
	username char(32),
	password char(32)
);

-- This table stores group associations.  A group must have one member to 
-- exist.
-- The user<->group mapping is m to n
-- Note that this is not currently fully supported.
CREATE TABLE IF NOT EXISTS group_assoc (
	groupname char(32),
	username  char(32)
);

-- Table storing scrap-group associations.
-- Scraps Not Appearing In This Table are unrestricted.
-- M-N mapping for group - scrap.
-- Note that this is not currently fully supported.
CREATE TABLE IF NOT EXISTS scrap_group (
	groupname char(32),
	scrap_id  char(32)
);
